@@ -51,6 +51,8 @@
 static FILE *outfile;
 static char *outfname = NULL;
 
+static int ascii_encoding_8_bit = 0;
+
 static void
 out_err(int err)
 {
@@ -637,7 +639,7 @@ static struct sdr_field type2[] =
       .strvals = modifier_unit_fields },
     { "percentage",		SDR_BOOLBIT,	21, 0, 1 },
     { "base_unit",		SDR_BITS,	22, 0, 8 },
-    { "modifier_unit",		SDR_BITS,	23, 0, 8 },
+    { "modifier_unit_code",	SDR_BITS,	23, 0, 8 },
     { "sensor_direction",	SDR_BITS,	24, 6, 2,
       .strvals = sensor_direction_fields },
     { "id_string_modifier",	SDR_BITS,	24, 4, 2,
@@ -832,6 +834,7 @@ add_variable(const char *name, const char *value)
 	var->name = strdup(name);
 	if (!var->name) {
 	    fprintf(stderr, "Out of memory\n");
+	    free(var);
 	    return ENOMEM;
 	}
 	var->next = NULL;
@@ -991,6 +994,8 @@ get_delim_str(char **rtokptr, char **rval, char **err)
 	if (rv) {
 	    char *newrv = malloc(strlen(rv) + strlen(val) + 1);
 	    if (!newrv) {
+		if (rv)
+		    free(rv);
 		*err = "Out of memory copying string";
 		return -1;
 	    }
@@ -1473,8 +1478,12 @@ ipmi_compile_sdr(FILE *f, unsigned int type,
 		err = get_delim_str(&tokptr, &sval, errstr);
 		if (err)
 		    goto out_err;
-		ipmi_set_device_string(sval, IPMI_ASCII_STR, strlen(sval),
-				       str, 0, &out_len);
+		if (!ascii_encoding_8_bit)
+			ipmi_set_device_string2(sval, IPMI_ASCII_STR, strlen(sval),
+						str, 0, &out_len, IPMI_STRING_OPTION_NONE);
+		else
+			ipmi_set_device_string2(sval, IPMI_ASCII_STR, strlen(sval),
+						str, 0, &out_len, IPMI_STRING_OPTION_8BIT_ONLY);
 		free(sval);
 		if (out_len > 1) {
 		    unsigned char *newsdr = realloc(sdr, sdr_len + out_len - 1);
@@ -1951,7 +1960,7 @@ static char *progname;
 static void
 help(void)
 {
-    fprintf(stderr, "%s [-r] [-o <outfile>] [-d] <input file>\n", progname);
+    fprintf(stderr, "%s [-r] [-8] [-o <outfile>] [-d] <input file>\n", progname);
     exit(1);
 }
 
@@ -1975,6 +1984,8 @@ main(int argc, char *argv[])
 	    break;
 	if (strcmp(argv[argn], "-r") == 0) {
 	    outraw = 1;
+	} else if (strcmp(argv[argn], "-8") == 0) {
+	    ascii_encoding_8_bit = 1;
 	} else if (strcmp(argv[argn], "-d") == 0) {
 	    decompile = 1;
 	} else if (strcmp(argv[argn], "-o") == 0) {
