@@ -262,7 +262,7 @@ uucp_mk_lock(sys_data_t *sys, const char *devname)
 	if ((fd = open(lck_file, O_RDONLY)) >= 0) {
 	    int n;
 
-	    n = read(fd, &buf, sizeof(buf));
+	    n = read(fd, &buf, sizeof(buf) - 1);
 	    close(fd);
 	    if( n == 4 ) 		/* Kermit-style lockfile. */
 		pid = buf.ival;
@@ -325,6 +325,7 @@ sol_to_termios_bitrate(ipmi_sol_t *sol, int solbps)
 	if (retried)
 	    return B9600;
 	solbps = sol->solparm.default_bitrate;
+	retried =1;
 	goto retry;
     }
 }
@@ -424,6 +425,7 @@ sol_serial_update_modemstate(ipmi_sol_t *sol, int ctspause, int deassert_dcd)
 	    val &= ~(TIOCM_DTR | TIOCM_RTS);
 	    val |= modemstate;
 	    ioctl(sol->soldata->fd, TIOCMSET, &val);
+	    sd->modemstate = modemstate;
 	}
     }
 }
@@ -438,14 +440,18 @@ sol_serial_activate(ipmi_sol_t *sol, msg_t *msg)
      * always monitor the history.
      */
     if (!sol->history_size) {
+	int modemstate;
+
+	ioctl(sd->fd, TIOCMGET, &modemstate);
 	if ((msg->data[2] & 1) == 0) {
 	    /* Assuming standard NULL modem, RTS->CTS, DTR->DSR/DCD */
-	    int modemstate;
-	    ioctl(sd->fd, TIOCMGET, &modemstate);
 	    modemstate |= TIOCM_DTR | TIOCM_RTS;
 	    sd->modemstate = TIOCM_DTR | TIOCM_RTS;
-	    ioctl(sd->fd, TIOCMSET, &modemstate);
+	} else {
+	    modemstate &= ~(TIOCM_DTR | TIOCM_RTS);
+	    sd->modemstate = 0;
 	}
+	ioctl(sd->fd, TIOCMSET, &modemstate);
     }
 
     return 0;
@@ -1155,7 +1161,7 @@ handle_sol_port_payload(lanserv_data_t *lan, ipmi_sol_t *sol, msg_t *msg)
     char isnack, isbreak, ctspause, deassert_dcd, flush_in, flush_out;
     unsigned char *data;
     unsigned int len;
-    int need_send_ack = 0;
+    int need_send_ack = 0, is_dup = 0;
     struct timeval tv;
 
     if (!sol->active || msg->len < 4)
@@ -1175,16 +1181,40 @@ handle_sol_port_payload(lanserv_data_t *lan, ipmi_sol_t *sol, msg_t *msg)
     data = msg->data + 4;
     len = msg->len - 4;
 
+    is_dup = seq == sd->last_acked_packet;
+
+    if (!is_dup) {
+	/* Don't re-do an operation that we already did. */
+
+	if (flush_out) {
+	    sd->waiting_ack = 0;
+	    next_seq(sd);
+	    sd->outlen = 0;
+	}
+
+	if (flush_in) {
+	    unsigned int oldlen = sd->inlen;
+
+	    sd->inlen = 0;
+	    if (oldlen == sizeof(sd->inbuf))
+		send_ack(sol);
+	}
+
+	if (isbreak)
+	    sd->send_break(sol);
+    }
+
+    /* Indepotent with respect to duplicates. */
+    sd->update_modemstate(sol, ctspause, deassert_dcd);
+
     if (seq != 0) {
-	if (seq == sd->last_acked_packet) {
+	if (is_dup) {
 	    need_send_ack = 1;
 	} else if (sd->fd == -1) {
 	    /* Ignore the data. */
-	    if  (len) {
-		sd->last_acked_packet = seq;
-		need_send_ack = 1;
-		sd->last_acked_packet_len = len;
-	    }
+	    sd->last_acked_packet = seq;
+	    need_send_ack = 1;
+	    sd->last_acked_packet_len = len;
 	} else if (len) {
 	    sd->last_acked_packet = seq;
 	    if (sol->do_telnet) {
@@ -1211,6 +1241,10 @@ handle_sol_port_payload(lanserv_data_t *lan, ipmi_sol_t *sol, msg_t *msg)
 	    sd->last_acked_packet_len = len;
 	    need_send_ack = 1;
 	    set_write_enable(sol->soldata);
+	} else {
+	    sd->last_acked_packet = seq;
+	    sd->last_acked_packet_len = 0;
+	    need_send_ack = 1;
 	}
     }
 
@@ -1244,20 +1278,6 @@ handle_sol_port_payload(lanserv_data_t *lan, ipmi_sol_t *sol, msg_t *msg)
 
     if (need_send_ack)
 	send_ack(sol);
-
-    if (flush_out) {
-	sd->waiting_ack = 0;
-	next_seq(sd);
-	sd->outlen = 0;
-    }
-
-    if (flush_in)
-	sd->inlen = 0;
-
-    if (isbreak)
-	sd->send_break(sol);
-
-    sd->update_modemstate(sol, ctspause, deassert_dcd);
 }
 
 static int
@@ -1427,6 +1447,7 @@ sol_write_ready(int fd, void *cb_data)
 {
     ipmi_sol_t *sol = cb_data;
     soldata_t *sd = sol->soldata;
+    unsigned int oldlen;
     int rv;
 
     rv = write(fd, sd->inbuf, sd->inlen);
@@ -1437,10 +1458,15 @@ sol_write_ready(int fd, void *cb_data)
 	sol_port_error(sol);
 	return;
     }
+    if (rv == 0)
+	return;
 
     if (((unsigned int) rv) < sd->inlen)
 	memcpy(sd->inbuf, sd->inbuf + rv, sd->inlen - rv);
+    oldlen = sd->inlen;
     sd->inlen -= rv;
+    if (oldlen == sizeof(sd->inbuf))
+	send_ack(sol);
 
     set_write_enable(sd);
 }
